library autoellipsis;
import 'package:flutter/material.dart';

class Ellipsis {

  static double _textWidth(String text, TextStyle style) {
    final TextPainter textPainter = TextPainter(
        text: TextSpan(text: text, style: style),
        maxLines: 1,
        textDirection: TextDirection.ltr)
      ..layout(minWidth: 0, maxWidth: double.infinity);


    return textPainter.width;
  }

  static String compactString(String text, TextStyle style, double maxWidth, {EllipsisFormat position = EllipsisFormat.End, bool byWord = true}) {
    if (text.isEmpty || text == null) {
      return text;
    }

    if (style == null) {
      return text;
    }

    if (position == EllipsisFormat.None) {
      return text;
    }

    if (maxWidth == null || maxWidth == double.infinity) {
      return text;
    }

    if (_textWidth(text, style) <= maxWidth) {
      return text;
    }
    List<String> words = byWord ? text.split(' ') : null;
    int cut = byWord ? words.length : text.length;
    int start = 0;
    int end = cut;
    bool grow = true;
    String fit;

    while (cut > 1) {
      cut -= cut ~/ 2;
      int mult = grow ? 1 : -1;

      switch (position) {
        case EllipsisFormat.End:
          start += mult * cut;
          break;
        case EllipsisFormat.Start:
          end -= mult * cut;
          break;
        default:
          start += mult * (cut ~/ 2);
          end -= mult * (cut - (cut ~/ 2));
          break;
      }

      fit = byWord ? '${words.getRange(0, start).join(' ')}...${words.getRange(
          end, words.length).join(' ')}' :
      '${text.substring(0, start)}...${text.substring(end, text.length)}';

      grow = _textWidth(fit, style) <= maxWidth ? true : false;
    }
    return fit;
  }

  static Text compactText(Text text, double maxWidth, {EllipsisFormat position = EllipsisFormat.End, bool byWord = true}){
    return new Text(
      compactString(text.data, text.style, maxWidth, position: position, byWord: byWord),
      key: text.key,
      style: text.style,
      strutStyle: text.strutStyle,
      textAlign: text.textAlign,
      textDirection: text.textDirection,
      locale: text.locale,
      softWrap: text.softWrap,
      overflow: text.overflow,
      textScaleFactor: text.textScaleFactor,
      maxLines: text.maxLines,
      semanticsLabel: text.semanticsLabel,
      textWidthBasis: text.textWidthBasis,
    );
  }

}

enum EllipsisFormat
{
  None,
  End,
  Start,
  Middle,
}
